--[[
**********************************************************************
MagicDKP Client - a client for the Magic DKP System.
**********************************************************************
This file is part of MagicDKP Client, a World of Warcraft Addon

MagicDKP Client is free software: you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

MagicDKP Client is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with MagicDKP Client.  If not, see <http://www.gnu.org/licenses/>.
**********************************************************************
]]

local MINOR_VERSION = tonumber(("$Revision: 25 $"):match("%d+"))

MDKP_Client = LibStub("AceAddon-3.0"):NewAddon("MagicDKP_Client", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
local mod = MDKP_Client
mod.MAJOR_VERSION = "MagicDKP_Client-1.0"
mod.MINOR_VERSION = MINOR_VERSION
MagicDKP_ClientDB = {}

local MagicComm = LibStub("MagicComm-1.0")
local media = LibStub("LibSharedMedia-3.0")
local currentEvent, currentSender, currentRaid

local networkData = {}
 pendingStandby = {}

function mod:OnEnable()
   MagicComm:RegisterListener(self, "MD")
   local popup = _G.StaticPopupDialogs
   if type(popup) ~= "table" then
      popup = {}
   end
   if type(popup["MDKPStandbyCheck"]) ~= "table" then
      popup["MDKPStandbyCheck"] = {
	 text = "",
	 button1 = "I am here!",
	 whileDead = 1,
	 hideOnEscape = 0,
	 timeout = 300,
	 startDelay = 0,
	 delayText = "Hello",
	 OnAccept = function() self:SendStandbyAck() self:ScheduleTimer("NextCheck", 2) end,
	 OnShow = function()
		     self:ScheduleRepeatingTimer("UpdatePopupTimer", 1, { time(), getglobal(this:GetName().."Text") })
		  end
      }
   end
   self:RegisterOptions();
end

function mod:OnDisable()
   MagicComm:UnregisterListener(self, "MD")
end

local function SetNetworkData(cmd, data, misc1, misc2, misc3, misc4)
   networkData.cmd = cmd
   networkData.data = data
   networkData.misc1 = misc1
   networkData.misc2 = misc2
   networkData.misc3 = misc3
   networkData.misc4 = misc4
end

function mod:SendUrgentMessage(channel, recipient)
   MagicComm:SendUrgentMessage(networkData, "MD", channel, recipient)
end

local standbyText = "%s has initiated a standby check for %s. Standby check expiring in %ds."

function mod:UpdatePopupTimer(data)
   local timeleft = 300 - (time()-data[1])
   if timeleft > 0 then
      data[2]:SetText(standbyText:format(currentSender, currentEvent, timeleft))
   else
      self:CancelAllTimers()
      StaticPopup_Hide("MDKPStandbyCheck")
      currentSender = nil
      currentEvent = nil
      currentRaid = nil
      self:ScheduleTimer("NextCheck", 2)
   end
end

function mod:OnStandbyCheck(event, sender, raidid)
   if currentEvent then
      pendingStandby[#pendingStandby+1] = {
	 sender = sender,
	 event = event,
	 raid = raidid,
      }
   else
      self:CancelAllTimers()
      mod:ShowStandbyCheckDialog(sender, event, raidid)
   end
end


function mod:ShowStandbyCheckDialog(sender, event, raidid)
   currentSender = sender
   currentEvent = event
   currentRaid = raidid
   StaticPopupDialogs["MDKPStandbyCheck"].text = standbyText:format(currentSender, currentEvent, 300)
   StaticPopup_Show("MDKPStandbyCheck")
   if not MagicDKP_ClientDB.sound then
      MagicDKP_ClientDB.sound = "Cartoon FX"
   end
   PlaySoundFile(media:Fetch("sound", MagicDKP_ClientDB.sound))
end

function mod:SendStandbyAck()
   self:CancelAllTimers()
   StaticPopup_Hide("MDKPStandbyCheck")
   if currentSender and currentEvent then
      SetNetworkData("STANDBYRESPONSE", UnitName("player"), currentEvent, currentRaid)
      self:SendUrgentMessage("WHISPER", currentSender)
      currentSender = nil
      currentEvent = nil
      currentRaid = nil
   end
end

function mod:NextCheck()
   if currentEvent then return end
   last = #pendingStandby
   if last > 0 then
      mod:ShowStandbyCheckDialog(pendingStandby[last].sender, 
				 pendingStandby[last].event, 
				 pendingStandby[last].raid)
      pendingStandby[last] = nil
   end
end

local config = {
   type = "group",
   name = "MagicDKP Client",
   args = {
      sound = {
         type = 'select',
         dialogControl = 'LSM30_Sound',
         name = 'Alert Sound',
         desc = 'Sound to play when',
         values = AceGUIWidgetLSMlists.sound,
         get = function() return MagicDKP_ClientDB.sound end,
         set = function(_,key) MagicDKP_ClientDB.sound = key end,
      }
   }
}
function mod:RegisterOptions()
   LibStub("AceConfig-3.0"):RegisterOptionsTable("MagicDKP Client", config)
   LibStub("AceConfigDialog-3.0"):AddToBlizOptions("MagicDKP Client", "MagicDKP Client")
end

-- /dump MDKP_Client:OnStandbyCheck("Event", "Helaani")
